from ..utils import dHf, Sf, Gf
from .species import Species


class CO(Species):
    def __init__(self, T_C, nu=None):
        # https://webbook.nist.gov/cgi/cbook.cgi?ID=C630080&Units=SI&Mask=1#Thermo-Gas
        super().__init__(T_C, nu)
        self.min_temp_K = 298
        self.max_temp_K = 1300
        self.check_temps()
        self.Hf_298 = -110.53  # kJ/mol
        self.thermo_params = {
            "A": 25.56759,
            "B": 6.096130,
            "C": 4.054656,
            "D": -2.671301,
            "E": 0.131021,
            "F": -118.0089,
            "G": 227.3665,
            "H": -110.5271,
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



from ..utils import dHf, Sf, Gf
from .species import Species


class H2(Species):
    def __init__(self, T_C, nu=None):
        super().__init__(T_C, nu)
        self.min_temp_K = 298
        self.max_temp_K = 1000
        self.check_temps()
        self.Hf_298 =  0 # kJ/mol
        self.thermo_params = {
            "A": 33.066178,
            "B": -11.363417,
            "C": 11.432816,
            "D": -2.772874,
            "E": -0.158558,
            "F": -9.980797,
            "G": 172.707974,
            "H": 0,
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



from ..utils import dHf, Sf, Gf
from .species import Species


class CH4(Species):
    def __init__(self, T_C, nu=None):
        super().__init__(T_C, nu)
        self.min_temp_K = 298
        self.max_temp_K = 1300
        self.check_temps()
        self.Hf_298 = -74.6 # kJ/mol
        self.thermo_params = {
            "A": -0.703029,
            "B": 108.4773,
            "C": -42.52157,
            "D": 5.862788,
            "E": 0.678565,
            "F": -76.84376,
            "G": 158.7163,
            "H": -74.87310
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



from ..utils import dHf, Sf, Gf
from .species import Species


class H2O(Species):
    def __init__(self, T_C, nu=None):
        super().__init__(T_C, nu)
        self.min_temp_K = 500
        self.max_temp_K = 1700
        self.check_temps()
        self.Hf_298 = -241.83  # kJ/mol
        self.thermo_params = {
            "A": 30.09200,
            "B": 6.832514,
            "C": 6.793435,
            "D": -2.534480,
            "E": 0.082139,
            "F": -250.8810,
            "G": 223.3967,
            "H": -241.8264
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



import plotly.graph_objects as go, numpy as np

from catrxneng.constants import *


class Reaction:
    def __init__(self):
        pass

    def dG_rxn(self):
        return self.dH_rxn() - self.T_K * self.dS_rxn()

    def Keq(self):
        return np.exp(-self.dG_rxn() / R["kJ/mol/K"] / self.T_K)

    def plot_K_vs_temp(self):
        self.T_K = self.T_C + 273
        fig = go.Figure()
        trace = go.Scatter(x=self.T_C, y=self.Keq(), mode="lines")
        fig.add_trace(trace)
        fig.update_layout(
            title=dict(text="<b>Equilibrium constant</b>", x=0.5),
            xaxis_title="<b>Temperature (°C)</b>",
            yaxis_title="<b>K</b>",
            width=800
        ) 
        fig.show()



import numpy as np
import plotly.graph_objects as go, numpy as np
from scipy.integrate import solve_ivp

from ..constants import *


class PFR:
    def __init__(self, w, P, whsv, y0, r):
        self.w = w  # gcat
        self.P = P  # bara
        self.whsv = whsv  # mmol/min/gcat
        self.y0 = y0
        self.Ft0 = whsv * w  # mmol/min
        self.F0 = y0 * self.Ft0  # mmol/min
        self.nmlm = self.F0 * mol_gas_vol["nL/mol"]  # normal mL/min
        self.total_nmlm = self.nmlm.sum()
        self.r = r  # mmol/min/gcat
        self.solve()

    def solve(self):
        def df(w, F):
            Ft = F.sum()
            y = F / Ft
            p = y * self.P
            dFdw = np.array([])
            for i in range(0, len(F)):
                ri = self.r[i](p)  # mass balance
                dFdw = np.append(dFdw, ri)
            return dFdw

        w_span = (0, self.w)
        w_eval = np.linspace(0, self.w, 100)
        solution = solve_ivp(df, w_span, self.F0, t_eval=w_eval)
        self.w = solution.t
        self.F = solution.y
        Ft = np.zeros(len(self.w))
        for Fi in self.F:
            Ft = Ft + Fi
        self.y = []
        for Fi in self.F:
            self.y.append(Fi / Ft)
        self.X = (self.F0[0] - self.F[0]) / self.F0[0] * 100

    def plot_molfrac_vs_w(self, labels):
        fig = go.Figure()
        for i, label in enumerate(labels):
            if label != "inert":
                trace = go.Scatter(x=self.w, y=self.y[i], mode="lines", name=label)
                fig.add_trace(trace)
        fig.update_layout(
            title=dict(text="<b>Mole fractions vs. catalyst mass</b>", x=0.5),
            xaxis_title="<b>Catalyst mass (g)</b>",
            yaxis_title="<b>Mole fraction</b>",
            width=700,
        )
        fig.show()

    def plot_conv_vs_w(self, label, Xeq=None):
        fig = go.Figure()
        trace = go.Scatter(x=self.w, y=self.X, mode="lines", name=label)
        fig.add_trace(trace)
        if Xeq:
            trace = go.Scatter(
                x=self.w,
                y=np.zeros(len(self.w)) + Xeq,
                mode="lines",
                name=f"Equilibrium {label}",
            )
            fig.add_trace(trace)
        fig.update_layout(
            title=dict(text="<b>Conversion vs. catalyst mass</b>", x=0.5),
            xaxis_title="<b>Catalyst mass (g)</b>",
            yaxis_title="<b>Conversion (%)</b>",
            width=800,
        )
        fig.show()



import numpy as np

class Species:
    def __init__(self, T_C, nu):
        self.T_C = T_C
        self.T_K = self.T_C + 273
        self.nu = nu

    def check_temps(self):
        if isinstance(self.T_K,np.ndarray):
            if self.T_K.min() < self.min_temp_K or self.T_K.max() > self.max_temp_K: 
                raise ValueError("Invalid temperature.")
        else:
            if self.T_K < self.min_temp_K or self.T_K > self.max_temp_K:
                raise ValueError("Invalid temperature.")




from .species import Species
from ..utils import dHf, Sf, Gf


class CO2(Species):
    def __init__(self, T_C,nu=None):
        super().__init__(T_C, nu)
        self.min_temp_K = 298
        self.max_temp_K = 1200
        self.check_temps()
        self.Hf_298 = -393.51  # kJ/mol
        self.thermo_params = {
            "A": 24.99735,
            "B": 55.18696,
            "C": -33.69137,
            "D": 7.948387,
            "E": -0.136638,
            "F": -403.6075,
            "G": 228.2431,
            "H": -393.5224,
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



from scipy.optimize import fsolve
from .reaction import Reaction
from ..species import CO2, H2, C2H4, H2O


class CO2FTS(Reaction):
    def __init__(self, temp_C):
        self.T_C = temp_C
        self.T_K = self.T_C + 273
        self.co2 = CO2(temp_C)
        self.h2 = H2(temp_C)
        self.c2h4 = C2H4(temp_C)
        self.h2o = H2O(temp_C)

    def dH_rxn(self):
        return self.c2h4.Hf() + 4 * self.h2o.Hf() - 2 * self.co2.Hf() - 6 * self.h2.Hf()

    def dS_rxn(self):
        return self.c2h4.Sf() + 4 * self.h2o.Sf() - 2 * self.co2.Sf() - 6 * self.h2.Sf()

    def Xeq(self, p0):
        def f(extent):
            Keq = (
                (p0[2] + extent)
                * (p0[3] + extent) ** 4
                / (p0[0] - extent) ** 2
                / (p0[1] - extent) ** 6
            )
            return Keq - self.Keq()

        extent = fsolve(f, 1)
        return extent / p0[0] * 100



from .reaction import Reaction
from ..species import CO, H2O, CO2, H2


class WGS(Reaction):
    def __init__(self,temp_C):
        self.T_C = temp_C
        self.T_K = self.T_C + 273
        self.co = CO(temp_C)
        self.co2 = CO2(temp_C)
        self.h2 = H2(temp_C)
        self.h2o = H2O(temp_C)

    def dH_rxn(self):
        return self.co2.Hf() + self.h2.Hf() - self.co.Hf() - self.h2o.Hf()

    def dS_rxn(self):
        return self.co2.Sf() + self.h2.Sf() - self.co.Sf() - self.h2o.Sf()


from ..utils import dHf, Sf, Gf
from .species import Species


class C2H4(Species):
    def __init__(self, T_C, nu=None):
        super().__init__(T_C, nu)
        self.min_temp_K = 298
        self.max_temp_K = 1200
        self.check_temps()
        self.Hf_298 = 52.4  # kJ/mol
        self.thermo_params = {
            "A": -6.387880,
            "B": 184.4019,
            "C": -112.9718,
            "D": 28.49593,
            "E": 0.315540,
            "F": 48.17332,
            "G": 163.1568,
            "H": 52.46694
        }

    def Hf(self):
        return self.Hf_298 + dHf(self)  # kJ/mol

    def Sf(self):
        return Sf(self)  # kJ/mol

    def Gf(self):
        return Gf(self)  # kJ/mol



import numpy as np


def dHf(species):
    t = species.T_K / 1000
    params = species.thermo_params
    return (
        params["A"] * t
        + params["B"] * t**2 / 2
        + params["C"] * t**3 / 3
        + params["D"] * t**4 / 4
        - params["E"] / t
        + params["F"]
        - params["H"]
    )  # kJ/mol


def Sf(species):
    t = species.T_K / 1000
    params = species.thermo_params
    return (
        params["A"] * np.log(t)
        + params["B"] * t
        + params["C"] * t**2 / 2
        + params["D"] * t**3 / 3
        - params["E"] / (2 * t**2)
        + params["G"]
    ) / 1000  # kJ/mol/K

def Gf(species):
    return species.Hf() - species.T_K * species.Sf() # kJ/mol



from setuptools import setup, find_packages

setup(
    name="catrxneng",
    version="1.0.9",
    packages=find_packages(),
    install_requires=[],
    author="Gordon Brezicki",
    description="A collection of Python utilities for catalysis and reaction engineering",
    url="https://github.com/gbrezicki1/catrxneng",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
)



from scipy.optimize import fsolve
import numpy as np
from .reaction import Reaction
from ..species import CO2, H2, CO, H2O


class RWGS(Reaction):
    def __init__(self, temp_C):
        self.T_C = temp_C
        self.T_K = self.T_C + 273
        self.co = CO(temp_C)
        self.co2 = CO2(temp_C)
        self.h2 = H2(temp_C)
        self.h2o = H2O(temp_C)

    def dH_rxn(self):
        return self.co.Hf() + self.h2o.Hf() - self.co2.Hf() - self.h2.Hf()

    def dS_rxn(self):
        return self.co.Sf() + self.h2o.Sf() - self.co2.Sf() - self.h2.Sf()

    def Xeq(self, p0):
        def f(extent):
            Keq = (
                (p0[2] + extent)
                * (p0[3] + extent)
                / (p0[0] - extent)
                / (p0[1] - extent)
            )
            return Keq - self.Keq()
        extent = fsolve(f, 1)
        return extent/p0[0] * 100

